#!/usr/bin/env python3
# vim:ts=4:sts=4:sw=4:expandtab

import argparse
import logging
import pathlib
import random
import socket
import struct
import traceback
import threading

DEFAULT_HOST='127.0.0.1'
DEFAULT_PORT=3456
DEFAULT_DOWNLOADS='./Downloads'

def client_communication(client_socket, server_address, downloads_path, request_path):
    try:
        downloads_dir = pathlib.Path(downloads_path)
        downloads_dir.mkdir(parents=True, exist_ok=True)
        download_path = downloads_dir / pathlib.Path(request_path).name
        trans_id = random.randint(0,2**64)
        #TODO: Reliably send request path to server
        request_path = bytes(request_path, 'utf-8')
        client_socket.sendto(struct.pack('!QQH', trans_id, 0, len(request_path))+request_path, server_address)
        logging.debug(f'Requested metadata of {request_path} from {server_address}')
        (response, response_address) = client_socket.recvfrom(2**16)
        trans_id, part_num, part_count = struct.unpack('!QQQ', response)
        logging.debug(f'Received metadata ({part_count} parts) of {request_path} from {server_address}')
        downloads_dir = pathlib.Path(downloads_path)
        downloads_dir.mkdir(parents=True, exist_ok=True)
        with download_path.open('wb') as request_file:
            for part_num in range(1,part_count+1):
                client_socket.sendto(struct.pack('!QQH', trans_id, part_num, len(request_path))+request_path, server_address)
                logging.debug(f'Requested part {part_num} of {request_path} from {server_address}')
                (response, response_address) = client_socket.recvfrom(2**16)
                response_id, response_part = struct.unpack('!QQ', response[0:16])
                logging.debug(f'Received part {response_part} of {request_path} from {server_address}')
                request_file.write(response[16:])
        logging.info(f'Receiving a file {request_path} from {server_address} succeeded')
    except:
        traceback.print_exc()
        logging.error(f'Receiving a file {request_path} from {server_address} failed')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='File Transfer TCP Client')
    parser.add_argument('--debug', action='store_true')
    parser.add_argument('--host', default=DEFAULT_HOST)
    parser.add_argument('--port', default=DEFAULT_PORT)
    parser.add_argument('--downloads', default=DEFAULT_DOWNLOADS)
    parser.add_argument('path')
    args = parser.parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    for family, type, proto, canonname, sockaddr in socket.getaddrinfo(args.host, args.port, family=socket.AF_INET, type=socket.SOCK_DGRAM):
        with socket.socket(family, type) as client_socket:
            client_communication(client_socket, sockaddr, args.downloads, args.path)
